<?php


namespace App\Modules\Book\Services;


use App\Http\Controllers\QuickApp\Book\Transformers\BookTransformer;
use App\Modules\Book\Models\BookConfig;
use App\Modules\RecommendBook\Services\QappRecommendService;

class BookAuditService
{
    /**
     * 审核数据
     * @param $sex
     * @param $package
     * @return array[]
     */
    public static function getHomeBooksData($sex, $package,$is_auth,$package_id): array
    {
        // 基本配置数据
        $home = config('home.default');
        if ($package === 'com.app.kyy.jdqyy') {
            $home = config('home.new');
        }

        if (!$is_auth){
            $home = config('home.ycsd');
        }

        // banner
        $banner      = $home['reco_banner'];
        $bannerBooks = $banner[$sex];
        if ($sex == 'male') {
            $channel          = 1;
        } else {
            $channel          = 2;
        }

        // 模块
        [$hot, $live, $recom, $new] = [$home['hot'], $home['zhibo'], $home['recom'], $home['new_recom']];
        [$hotBids, $liveBids, $recomBids, $newBids] = BookConfigService::HomePageRecomBookCheck($hot, $live, $recom, $new,$package_id, $package,$channel);

        // 一次性获取书籍列表
        $bids  = array_merge($hotBids, $liveBids, $recomBids, $newBids);

        $channel_id = is_public_package($package) ? get_default_public_channel_id() : 0;
        $books = BookConfigService::getBookLists(compact('bids','channel_id'));
        return [
            [
                'type'  => 'reco_banner',
                'lable' => $banner['label'],
                'books' => $bannerBooks
            ],
            [
                'type'  => 'hot',
                'lable' => $hot['label'],
                'books' => collectionTransform(new BookTransformer, collect($books)->whereIn('bid', $hotBids)->all())
            ],
            [
                'type'  => 'zhibo',
                'lable' => $live['label'],
                'books' => collectionTransform(new BookTransformer, collect($books)->whereIn('bid', $liveBids)->all())
            ],
            [
                'type'  => 'recom',
                'lable' => $recom['label'],
                'books' => collectionTransform(new BookTransformer, collect($books)->whereIn('bid', $recomBids)->all())
            ],
            [
                'type'  => 'new_recom',
                'lable' => $new['label'],
                'books' => collectionTransform(new BookTransformer, collect($books)->whereIn('bid', $newBids)->all())
            ],
        ];
    }

    public static function getYueaiHomeBooksData($sex, $package,$is_auth,$channel,$package_id): array
    {
        $home = config('home.yueai');
        if (!$is_auth){
            $home = config('home.ycsd');
        }
        $banner      = $home['reco_banner'];
        $bannerBooks = $banner[$sex];

        // 模块
        [$hot, $live, $recom, $new] = [$home['hot'], $home['zhibo'], $home['recom'], $home['new_recom']];
        [$hotBids, $liveBids, $recomBids, $newBids] =  BookConfigService::HomePageRecomBookCheck($hot, $live, $recom, $new,$package_id, $package,$channel,false);

        return array_filter([
            ['type' => 'reco_banner', 'lable' => '首页banner', 'books' => $bannerBooks],
            ['type' => 'hot', 'lable' => '热门书单', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($hotBids,['recommend_index','desc'],false))],
            ['type' => 'zhibo', 'lable' => '神书直播', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($liveBids,['recommend_index','desc'],false))],
            ['type' => 'recom', 'lable' => '小编精选', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($recomBids,['recommend_index','desc'],false))],
            ['type' => 'new_recom', 'lable' => '人气新书', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($newBids,['recommend_index','desc'],false))],
        ]);

    }

    private static function getCheckBooks($bid_list,$channel,$package,$is_author)
    {
        $hidden_cp = getHiddenCp($package);
//        if(!is_public_package($package)){
//            $hidden_cp = array_merge($hidden_cp,['lianshang']);
//        }
        //获取书本数量
        $count = count($bid_list);
        $where = [
            ['book_configs.charge_type','!=','BOOK'],
        ];
        //获取当前有效书本数量
        $book_count = BookConfig::join('books', 'book_configs.bid', '=', 'books.id')
            ->leftjoin('book_categories', 'books.category_id', 'book_categories.id')
            ->whereIn('book_configs.bid',$bid_list)
            ->whereIn('book_configs.is_on_shelf',[1,2])
            ->whereNotIn('book_configs.cp_source',$hidden_cp)
            ->where($where)
            ->where('book_categories.pid',$channel)
            ->count();
        if($count == $book_count){
            return $bid_list;
        }
        //获取需要补充的书籍数量
        $supplement_count = (($count - $book_count) > 0) ? $count - $book_count : 0;
        if($supplement_count <= 0){
            return $bid_list;
        }
        //获取书籍交集bid，过滤掉不符合要求的书
        $bids = BookConfig::join('books', 'book_configs.bid', '=', 'books.id')
            ->leftjoin('book_categories', 'books.category_id', 'book_categories.id')
            ->whereIn('book_configs.bid',$bid_list)
            ->whereIn('book_configs.is_on_shelf',[1,2])
            ->where($where)
            ->whereNotIn('book_configs.cp_source',$hidden_cp)
            ->where('book_categories.pid',$channel)
            ->pluck('book_configs.bid')->all();
        $bid_list = array_intersect($bid_list,$bids);

        //获取随机的有效的书籍bid
        $rand_bid = BookConfig::join('books', 'book_configs.bid', '=', 'books.id')
            ->leftjoin('book_categories', 'books.category_id', 'book_categories.id')
            ->where('book_configs.is_on_shelf',2)
            // ->where('book_configs.charge_type','!=','BOOK')
            ->where($where)
            ->whereNotIn('book_configs.cp_source',$hidden_cp)

            ->where('book_categories.pid',$channel)
            ->orderBy('book_configs.recommend_index', 'desc')
            ->limit($supplement_count)
            ->get()->pluck('bid')->toArray();

        return array_filter(array_merge($bid_list,$rand_bid));
    }

    public static function getHuPoHomeBooksData($channel,$books,$package_id,$package): array
    {
        $isAuthor = check_qapp_auth($package_id,1);
        $hotBids   = [65741,65743,65742,65744,66474,66475];
        $liveBids  = [66476,66477,66479,66480,66631,61596];
        $recomBids = QappRecommendService::getRecommendByPacketId($channel, 'recom',$package_id);
        $newBids   = QappRecommendService::getRecommendByPacketId($channel, 'new_recom',$package_id);
        [$hotBids, $liveBids, $recomBids, $newBids] =  BookConfigService::HomePageRecomBookCheck($hotBids, $liveBids, $recomBids, $newBids,$package_id, $package,$channel,false);

        return array_filter([
            ['type' => 'reco_banner', 'lable' => '首页banner', 'books' => $books],
            ['type' => 'hot', 'lable' => '热门书单', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds(self::getCheckBooks($hotBids,$channel,$package,$isAuthor),[],false))],
            ['type' => 'zhibo', 'lable' => '神书直播', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds(self::getCheckBooks($liveBids,$channel,$package,$isAuthor),[],false))],
            ['type' => 'recom', 'lable' => '小编精选', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($recomBids))],
            ['type' => 'new_recom', 'lable' => '人气新书', 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($newBids))]
        ]);
    }

    public static function getRecommendBooksNew($package_id, $package, $channel, $is_auth, $is_audit)
    {
        $config = config("home.audit")[$package];
        if($config){
           if ($channel == 1){
               $bids = $config['male'];
           }else {
               $bids = $config['female'];
           }
            [$hotBids, $liveBids, $recomBids, $newBids]  =  array_chunk($bids, 6);

        }else{
            if ($channel == 1){
                return self::getHomeBooksData('male',$package,$is_auth,$package_id);
            }else {
                return self::getHomeBooksData('female',$package,$is_auth,$package_id);
            }
        }
        [$hotBids, $liveBids, $recomBids, $newBids] = BookConfigService::HomePageRecomBookCheck($hotBids, $liveBids, $recomBids, $newBids,$package_id, $package,$channel);
        $list = [];
        foreach ($config['labels'] as $key =>  $v){
           $list[] =   ['type' => 'rec_'.$key, 'lable' => $v, 'books' => collectionTransform(new BookTransformer, BookConfigService::getBooksByIds($newBids))];
        }
        return  array_filter($list);

    }
}